`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2022/04/09 22:15:06
// Design Name: 
// Module Name: alu, adder
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////
`include "utils.v"

module alu(
        input       [31:0]  a,
        input       [31:0]  b,
        input       [4:0]   aluop,
        output reg  [31:0]  res,
        output reg  [1:0]   cmp
    );
    
    always @(*) begin
        case (aluop)
            `aluop_add:     res = a + b;
            `aluop_sub:     res = a - b;
            `aluop_lts:     res = $signed(a) < $signed(b) ? 32'b1 : 32'b0;
            `aluop_ltu:     res = $unsigned(a) < $unsigned(b) ? 32'b1 : 32'b0;
            `aluop_sll:     res = a << b[4:0];
            `aluop_srl:     res = a >> b[4:0];
            `aluop_sra:     res = $signed(a) >>> b[4:0];
            `aluop_xor:     res = a ^ b; 
            `aluop_or:      res = a | b; 
            `aluop_and:     res = a & b; 
            `aluop_mul:     res = a * b; 
            `aluop_mulh:    res = ($signed(a) * $signed(b)) >> 32; 
            `aluop_mulhsu:  res = ($signed(a) * $unsigned(b)) >> 32;
            `aluop_mulhu:   res = ($unsigned(a) * $unsigned(b)) >> 32;   
            `aluop_div:     res = $signed(a) / $signed(b); 
            `aluop_divu:    res = $unsigned(a) / $unsigned(b); 
            `aluop_rem:     res = $signed(a) % $signed(b); 
            `aluop_remu:    res = $unsigned(a) % $unsigned(b); 
            `aluop_lui:     res = b;      
            `aluop_eq:      res = a == b ? 32'b1: 32'b0; 
            `aluop_ne:      res = a != b ? 32'b1: 32'b0; 
            `aluop_ges:     res = $signed(a) >= $signed(b) ? 32'b1 : 32'b0; 
            `aluop_geu:     res = $unsigned(a) >= $unsigned(b) ? 32'b1 : 32'b0; 
            `aluop_jalr:    res = a + b;      
        endcase
        if (aluop == `aluop_jalr) cmp = 2'b11;
        else cmp = res != 32'b0 ? 2'b11 : 2'b00;
    end
endmodule

module adder(
        input       [31:0]  a,
        input       [31:0]  b,
        output reg  [31:0]  res
    );
    
    always @(*) begin
        res = a + b;
    end
endmodule
