`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2022/04/13 18:26:31
// Design Name: 
// Module Name: controlUnit
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////
`include "utils.v"

module controlUnit(
        input       [6:0]   func7,
        input       [6:0]   op,
        input       [2:0]   func3,
        output  reg [1:0]   buswsrc,
        output  reg         regwr,
        output  reg [2:0]   immctr,
        output  reg         alusrc,
        output  reg [4:0]   aluop,
        output  reg [1:0]   branch,
        output  reg         memwr,
        output  reg         memtoreg
    );
    
    always @(*) begin
        case(op)
            `opcode_R_type: begin
                buswsrc = 2'b01;
                regwr = `Enable;
                alusrc = 1'b0;
                // aluop TBD
                branch = 2'b0;
                memwr = `Disable;
                memtoreg = 1'b0;
                case(func7)
                    `funct7_add: begin // add/sll/slt/sltu/xor/srl/or/and
                        case(func3)
                            `funct3_add:    aluop = `aluop_add;
                            `funct3_sll:    aluop = `aluop_sll;
                            `funct3_slt:    aluop = `aluop_lts;
                            `funct3_sltu:   aluop = `aluop_ltu;
                            `funct3_xor:    aluop = `aluop_xor;
                            `funct3_srl:    aluop = `aluop_srl;
                            `funct3_or:     aluop = `aluop_or;
                            `funct3_and:    aluop = `aluop_and;
                        endcase
                    end
                    `funct7_sub: begin // sub/sra
                        case(func3)
                            `funct3_sub:    aluop = `aluop_sub;
                            `funct3_sra:    aluop = `aluop_sra;
                        endcase
                    end
                    `funct7_mul: begin // mul/div/rem (RV32M)
                        case(func3)
                            `funct3_mul:    aluop = `aluop_mul;
                            `funct3_mulh:   aluop = `aluop_mulh;
                            `funct3_mulhsu: aluop = `aluop_mulhsu;
                            `funct3_mulhu:  aluop = `aluop_mulhu;
                            `funct3_div:    aluop = `aluop_div;
                            `funct3_divu:   aluop = `aluop_divu;
                            `funct3_rem:    aluop = `aluop_rem;
                            `funct3_remu:   aluop = `aluop_remu;
                        endcase
                    end
                endcase
            end
            `opcode_I_calc: begin
                buswsrc = 2'b01;
                regwr = `Enable;
                immctr = `immctr_I;
                alusrc = 1'b1;
                // aluop TBD
                branch = 2'b0;
                memwr = `Disable;
                memtoreg = 1'b0;
                case (func3)
                    `funct3_addi:   aluop = `aluop_add;
                    `funct3_slti:   aluop = `aluop_lts;
                    `funct3_sltiu:  aluop = `aluop_ltu;
                    `funct3_xori:   aluop = `aluop_xor;
                    `funct3_ori:    aluop = `aluop_or;
                    `funct3_andi:   aluop = `aluop_and;
                    `funct3_slli:   aluop = `aluop_sll;
                    `funct3_srli:   aluop = func7[5] == 1'b0 ? `aluop_srl : `aluop_sra; // srl/sra
                endcase
            end
            `opcode_B_type: begin
                regwr = `Disable;
                immctr = `immctr_B;
                alusrc = 1'b0;
                // aluop TBD
                branch = 2'b01;
                memwr = `Disable;
                case (func3)
                    `funct3_beq:    aluop = `aluop_eq;
                    `funct3_bne:    aluop = `aluop_ne;
                    `funct3_blt:    aluop = `aluop_lts;
                    `funct3_bge:    aluop = `aluop_ges;
                    `funct3_bltu:   aluop = `aluop_ltu;
                    `funct3_bgeu:   aluop = `aluop_geu;
                endcase
            end
            `opcode_lw: begin
                buswsrc = 2'b01;
                regwr = `Enable;
                immctr = `immctr_I;
                alusrc = 1'b1;
                aluop = `aluop_add;
                branch = 2'b0;
                memwr = `Disable;
                memtoreg = 1'b1;
            end
            `opcode_sw: begin
                regwr = `Disable;
                immctr = `immctr_S;
                alusrc = 1'b1;
                aluop = `aluop_add;
                branch = 2'b0;
                memwr = `Enable;
            end
            `opcode_lui: begin
                buswsrc = 2'b01;
                regwr = `Enable;
                immctr = `immctr_U;
                alusrc = 1'b1;
                aluop = `aluop_lui;
                branch = 2'b0;
                memwr = `Disable;
                memtoreg = 1'b0;
            end
            `opcode_auipc: begin
                buswsrc = 2'b10;
                regwr = `Enable;
                immctr = `immctr_U;
                branch = 2'b0;
                memwr = `Disable;
                memtoreg = 1'b0;
            end
            `opcode_jal: begin
                buswsrc = 2'b00;
                regwr = `Enable;
                immctr = `immctr_J;
                aluop = `aluop_jalr;
                branch = 2'b01;
                memwr = `Disable;
                memtoreg = 1'b0;
            end
            `opcode_jalr: begin
                buswsrc = 2'b00;
                regwr = `Enable;
                immctr = `immctr_I;
                alusrc = 1'b1;
                aluop = `aluop_jalr;
                branch = 2'b10;
                memwr = `Disable;
                memtoreg = 1'b0;
            end
            default: begin
                buswsrc = 2'b00;
                regwr = `Disable;
                immctr = `immctr_I;
                alusrc = 1'b0;
                aluop = `aluop_add;
                branch = 2'b00;
                memwr = `Disable;
                memtoreg = 1'b0;
            end
        endcase
    end
endmodule
