`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2022/04/13 17:45:57
// Design Name: 
// Module Name: cpu_top
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////


module cpu_top(
        input   wire            clk,
        output  wire    [31:0]  result
    );
    
    wire [31:0] muxpc_to_pc;
    wire [31:0] pc_out;
    wire [31:0] instr;
    wire [6:0] funct7;
    wire [6:0] op;
    wire [2:0] funct3;
    wire [4:0] rs2;
    wire [4:0] rs1;
    wire [4:0] rd;
    wire [31:0] immin;
    wire [31:0] immout;
    
    wire [1:0] buswsrc;
    wire regwr;
    wire [2:0] immctr;
    wire alusrc;
    wire [4:0] aluop;
    wire [1:0] branch;
    wire memwr;
    wire memtoreg;
    
    wire [31:0] busa;
    wire [31:0] busb;
    wire [31:0] busw;
    
    wire [31:0] muxsrcb_alu;
    
    wire [31:0] alures;
    wire [1:0] alucmp;
    wire [1:0] pcsrc;
    
    wire [31:0] datard;
    
    wire [31:0] mux_datamem_regf;
    
    wire [31:0] const4;
    assign const4 = 32'H4;
    wire [31:0] adder4res;
    wire [31:0] adderores;
    
    assign result = mux_datamem_regf;
    
    // PC
    pc PC(
        .clk(clk), 
        .nxtpc(muxpc_to_pc), 
        .curpc(pc_out)
    );
    
    // instruct memory
    instructMemory instrmem(
//        .clk(clk),
        .addr(pc_out), 
        .instr(instr)
    );
    
    // instruct decoder
    id ID(
        .instr(instr), 
        .funct7(funct7), 
        .op(op), 
        .funct3(funct3), 
        .rs2(rs2), 
        .rs1(rs1), 
        .rd(rd), 
        .imm(immin)
    );
    
    // control unit
    controlUnit cu(
        .func7(funct7),
        .op(op),
        .func3(funct3),
        .buswsrc(buswsrc),
        .regwr(regwr),
        .immctr(immctr),
        .alusrc(alusrc),
        .aluop(aluop),
        .branch(branch),
        .memwr(memwr),
        .memtoreg(memtoreg)
    );
    
    // register file
    registerFile regf(
        .clk(clk),
        .rs1(rs1),
        .rs2(rs2),
        .rd(rd),
        .regwr(regwr),
        .busw(busw),
        .busa(busa),
        .busb(busb)
    );
    
    // immediate concatenator
    immediateCon immediate(
        .immCtr(immctr),
        .instr(immin),
        .immediate(immout)
    );
    
    // ALU
    alu ALU(
        .a(busa),
        .b(muxsrcb_alu),
        .aluop(aluop),
        .res(alures),
        .cmp(alucmp)
    );
    
    // pcsrc andgate
    pcsrcgate andgate(
        .branch(branch),
        .cmp(alucmp),
        .pcsrc(pcsrc)
    );
    
    // data memory
    dataMemory datamem(
        .clk(clk),
        .addr(alures),
        .wd(busb),
        .memwr(memwr),
        .rd(datard)
    );
    
    // mux at srcb
    mux muxsrcb(
        .muxinput0(busb),
        .muxinput1(immout),
        .muxop(alusrc),
        .muxoutput(muxsrcb_alu)
    );
    
    // mux at memtoreg
    mux muxmemreg(
        .muxinput0(alures),
        .muxinput1(datard),
        .muxop(memtoreg),
        .muxoutput(mux_datamem_regf)
    );
    
    // mux at busw
    mux3 muxbusw(
        .muxinput0(adder4res),
        .muxinput1(mux_datamem_regf),
        .muxinput2(adderores),
        .muxop(buswsrc),
        .muxoutput(busw)
    );
    
    // mux at pc
    mux3 muxpc(
        .muxinput0(adder4res),
        .muxinput1(adderores),
        .muxinput2(mux_datamem_regf),
        .muxop(pcsrc),
        .muxoutput(muxpc_to_pc)
    );
    
    // adder at pc+4
    adder pcadder4(
        .a(const4),
        .b(pc_out),
        .res(adder4res)
    );
    
    // adder at pc+offset
    adder pcaddero(
        .a(immout),
        .b(pc_out),
        .res(adderores)
    );
    
endmodule
