`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date: 2022/04/09 20:41:44
// Design Name: 
// Module Name: utils
// Project Name: 
// Target Devices: 
// Tool Versions: 
// Description: 
// 
// Dependencies: 
// 
// Revision:
// Revision 0.01 - File Created
// Additional Comments:
// 
//////////////////////////////////////////////////////////////////////////////////

// define utilities for the project, such as macro of opcodes.

// bool const

`define Enable  1'b1
`define Disable 1'b0

// opcode

`define opcode_lui      7'b0110111 // LUI (U)
`define opcode_auipc    7'b0010111 // AUIPC (U)
`define opcode_jal      7'b1101111 // JAL (J)
`define opcode_jalr     7'b1100111 // JALR (I)
`define opcode_B_type   7'b1100011 // (B)
`define opcode_beq      7'b1100011 // BEQ (B)
`define opcode_bne      7'b1100011 // BNE (B)
`define opcode_blt      7'b1100011 // BLT (B)
`define opcode_bge      7'b1100011 // BGE (B)
`define opcode_bltu     7'b1100011 // BLTU (B)
`define opcode_bgeu     7'b1100011 // BGEU (B)
`define opcode_lw       7'b0000011 // LW (I)
`define opcode_sw       7'b0100011 // SW (S)
`define opcode_I_calc   7'b0010011 // (I) calc
`define opcode_addi     7'b0010011 // ADDI (I)
`define opcode_slti     7'b0010011 // SLTI (I)
`define opcode_sltiu    7'b0010011 // SLTIU (I)
`define opcode_xori     7'b0010011 // XORI (I)
`define opcode_ori      7'b0010011 // ORI (I)
`define opcode_andi     7'b0010011 // ANDI (I)
`define opcode_slli     7'b0010011 // SLLI (I)
`define opcode_srli     7'b0010011 // SRLI (I)
`define opcode_srai     7'b0010011 // SRAI (I)
`define opcode_R_type   7'b0110011 // (R)
`define opcode_add      7'b0110011 // ADD (R)
`define opcode_sub      7'b0110011 // SUB (R)
`define opcode_sll      7'b0110011 // SLL (R)
`define opcode_slt      7'b0110011 // SLT (R)
`define opcode_sltu     7'b0110011 // SLTU (R)
`define opcode_xor      7'b0110011 // XOR (R)
`define opcode_srl      7'b0110011 // SRL (R)
`define opcode_sra      7'b0110011 // SRA (R)
`define opcode_or       7'b0110011 // OR (R)
`define opcode_and      7'b0110011 // AND (R)
`define opcode_mul      7'b0110011 // MUL (R)
`define opcode_mulh     7'b0110011 // MULH (R)
`define opcode_mulhsu   7'b0110011 // MULHSU (R)
`define opcode_mulhu    7'b0110011 // MULHU (R)
`define opcode_div      7'b0110011 // DIV (R)
`define opcode_divu     7'b0110011 // DIVU (R)
`define opcode_rem      7'b0110011 // REM (R)
`define opcode_remu     7'b0110011 // REMU (R)

// funct3

`define funct3_jalr     3'b000
`define funct3_beq      3'b000
`define funct3_bne      3'b001
`define funct3_blt      3'b100
`define funct3_bge      3'b101
`define funct3_bltu     3'b110
`define funct3_bgeu     3'b111
`define funct3_lw       3'b010
`define funct3_sw       3'b010
`define funct3_addi     3'b000
`define funct3_slti     3'b010
`define funct3_sltiu    3'b011
`define funct3_xori     3'b100
`define funct3_ori      3'b110
`define funct3_andi     3'b111
`define funct3_slli     3'b001
`define funct3_srli     3'b101
`define funct3_srai     3'b101
`define funct3_add      3'b000
`define funct3_sub      3'b000
`define funct3_sll      3'b001
`define funct3_slt      3'b010
`define funct3_sltu     3'b011
`define funct3_xor      3'b100
`define funct3_srl      3'b101
`define funct3_sra      3'b101
`define funct3_or       3'b110
`define funct3_and      3'b111
`define funct3_mul      3'b000
`define funct3_mulh     3'b001
`define funct3_mulhsu   3'b010
`define funct3_mulhu    3'b011
`define funct3_div      3'b100
`define funct3_divu     3'b101
`define funct3_rem      3'b110
`define funct3_remu     3'b111

// funct7

`define funct7_slli     7'b0000000
`define funct7_srli     7'b0000000
`define funct7_srai     7'b0100000
`define funct7_add      7'b0000000
`define funct7_sub      7'b0100000
`define funct7_sll      7'b0000000
`define funct7_slt      7'b0000000
`define funct7_sltu     7'b0000000
`define funct7_xor      7'b0000000
`define funct7_srl      7'b0000000
`define funct7_sra      7'b0100000
`define funct7_or       7'b0000000
`define funct7_and      7'b0000000
`define funct7_mul      7'b0000001
`define funct7_mulh     7'b0000001
`define funct7_mulhsu   7'b0000001
`define funct7_mulhu    7'b0000001
`define funct7_div      7'b0000001
`define funct7_divu     7'b0000001
`define funct7_rem      7'b0000001
`define funct7_remu     7'b0000001

// immctr

`define immctr_I        3'b000
`define immctr_S        3'b001
`define immctr_B        3'b010
`define immctr_U        3'b011
`define immctr_J        3'b100

// alu_op_code

`define aluop_add       5'D0
`define aluop_sub       5'D1
`define aluop_lts       5'D2
`define aluop_ltu       5'D3
`define aluop_sll       5'D4
`define aluop_srl       5'D5
`define aluop_sra       5'D6
`define aluop_xor       5'D7
`define aluop_or        5'D8
`define aluop_and       5'D9
`define aluop_mul       5'D10
`define aluop_mulh      5'D11
`define aluop_mulhsu    5'D12
`define aluop_mulhu     5'D13
`define aluop_div       5'D14
`define aluop_divu      5'D15
`define aluop_rem       5'D16
`define aluop_remu      5'D17
`define aluop_lui       5'D18
`define aluop_eq        5'D19
`define aluop_ne        5'D20
`define aluop_ges       5'D21
`define aluop_geu       5'D22
`define aluop_jalr      5'D23
